@@ -8,6 +8,7 @@ import sys
 import threading
 
 PLUGIN_PREFIX = "eql_"
+CASE_INSENSITIVE = True
 _loaded_plugins = False
 
 # Python2 and Python3 compatible type checking
@@ -54,6 +55,18 @@ def is_array(a):
     return isinstance(a, (list, tuple))
 
 
+def is_insensitive():
+    """Check if insensitivity is enabled."""
+    return CASE_INSENSITIVE
+
+
+def fold_case(s):
+    """Helper function for normalizing case for strings."""
+    if is_insensitive() and is_string(s):
+        return s.lower()
+    return s
+
+
 def str_presenter(dumper, data):
     """Patch YAML so that it folds the long query strings."""
     if len(data.splitlines()) > 1:  # check for multiline string
